#ifndef __WCEMMSYSTEM_H_
#define __WCEMMSYSTEM_H_

#include "sys/wcetypes.h"

#define WHDR_DONE 1
#define WHDR_PREPARED 2
#define WHDR_BEGINLOOP 4
#define WHDR_ENDLOOP 8
#define WHDR_INQUEUE 16

#define MMSYSERR_BASE 0
#define MMSYSERR_NOERROR 0
#define MMSYSERR_ERROR (MMSYSERR_BASE+1)
#define MMSYSERR_BADDEVICEID (MMSYSERR_BASE+2)
#define MMSYSERR_NOTENABLED (MMSYSERR_BASE+3)
#define MMSYSERR_ALLOCATED (MMSYSERR_BASE+4)
#define MMSYSERR_INVALHANDLE (MMSYSERR_BASE+5)
#define MMSYSERR_NODRIVER (MMSYSERR_BASE+6)
#define MMSYSERR_NOMEM (MMSYSERR_BASE+7)
#define MMSYSERR_NOTSUPPORTED (MMSYSERR_BASE+8)
#define MMSYSERR_BADERRNUM (MMSYSERR_BASE+9)
#define MMSYSERR_INVALFLAG (MMSYSERR_BASE+10)
#define MMSYSERR_INVALPARAM (MMSYSERR_BASE+11)
#define MMSYSERR_HANDLEBUSY (MMSYSERR_BASE+12)
#define MMSYSERR_INVALIDALIAS (MMSYSERR_BASE+13)
#define MMSYSERR_BADDB (MMSYSERR_BASE+14)
#define MMSYSERR_KEYNOTFOUND (MMSYSERR_BASE+15)
#define MMSYSERR_READERROR (MMSYSERR_BASE+16)
#define MMSYSERR_WRITEERROR (MMSYSERR_BASE+17)
#define MMSYSERR_DELETEERROR (MMSYSERR_BASE+18)
#define MMSYSERR_VALNOTFOUND (MMSYSERR_BASE+19)
#define MMSYSERR_NODRIVERCB (MMSYSERR_BASE+20)
#define MMSYSERR_LASTERROR (MMSYSERR_BASE+20)

#define CALLBACK_TYPEMASK 0x70000
#define CALLBACK_NULL 0
#define CALLBACK_WINDOW 0x10000
#define CALLBACK_TASK 0x20000
#define CALLBACK_FUNCTION 0x30000
#define CALLBACK_THREAD CALLBACK_TASK
#define CALLBACK_EVENT 0x50000

#define WAVERR_BASE 32
#define WAVERR_BADFORMAT (WAVERR_BASE + 0)
#define WAVERR_STILLPLAYING (WAVERR_BASE + 1)
#define WAVERR_UNPREPARED (WAVERR_BASE + 2)
#define WAVERR_SYNC (WAVERR_BASE + 3)
#define WAVERR_LASTERROR (WAVERR_BASE + 3)

#define MM_WOM_OPEN 0x3BB
#define MM_WOM_CLOSE 0x3BC
#define MM_WOM_DONE 0x3BD
#define MM_WIM_OPEN 0x3BE
#define MM_WIM_CLOSE 0x3BF
#define MM_WIM_DATA 0x3C0
#define MM_MIM_OPEN 0x3C1
#define MM_MIM_CLOSE 0x3C2
#define MM_MIM_DATA 0x3C3
#define MM_MIM_LONGDATA 0x3C4
#define MM_MIM_ERROR 0x3C5
#define MM_MIM_LONGERROR 0x3C6
#define MM_MOM_OPEN 0x3C7
#define MM_MOM_CLOSE 0x3C8
#define MM_MOM_DONE 0x3C9

#define WOM_OPEN MM_WOM_OPEN
#define WOM_CLOSE MM_WOM_CLOSE
#define WOM_DONE MM_WOM_DONE
#define WIM_OPEN MM_WIM_OPEN
#define WIM_CLOSE MM_WIM_CLOSE
#define WIM_DATA MM_WIM_DATA

#define WAVE_MAPPER ((UINT)-1)

#define WAVE_FORMAT_1M08 1
#define WAVE_FORMAT_1S08 2
#define WAVE_FORMAT_1M16 4
#define WAVE_FORMAT_1S16 8
#define WAVE_FORMAT_2M08 16
#define WAVE_FORMAT_2S08 32
#define WAVE_FORMAT_2M16 64
#define WAVE_FORMAT_2S16 128
#define WAVE_FORMAT_4M08 256
#define WAVE_FORMAT_4S08 512
#define WAVE_FORMAT_4M16 1024
#define WAVE_FORMAT_4S16 2048
#define WAVE_FORMAT_PCM 1

typedef HANDLE HWAVEOUT;
typedef HWAVEOUT *LPHWAVEOUT;
typedef UINT MMRESULT;

typedef struct mmtime_tag {
   UINT wType;
   union {
       DWORD ms;
       DWORD sample;
       DWORD cb;
       DWORD ticks;
       struct {
           BYTE hour;
           BYTE min;
           BYTE sec;
           BYTE frame;
           BYTE fps;
           BYTE dummy;
           BYTE pad[2];
       } smpte;
       struct {
           DWORD songptrpos;
       } midi;
   } u;
} MMTIME,*PMMTIME,*LPMMTIME;

typedef struct wavehdr_tag {
   LPSTR lpData;
   DWORD dwBufferLength;
   DWORD dwBytesRecorded;
   DWORD dwUser;
   DWORD dwFlags;
   DWORD dwLoops;
   struct wavehdr_tag *lpNext;
   DWORD reserved;
} WAVEHDR,*PWAVEHDR,*LPWAVEHDR;

typedef struct waveformat_tag {
   WORD wFormatTag;
   WORD nChannels;
   DWORD nSamplesPerSec;
   DWORD nAvgBytesPerSec;
   WORD nBlockAlign;
} WAVEFORMAT,*PWAVEFORMAT,*LPWAVEFORMAT;

typedef struct tWAVEFORMATEX {
   WORD wFormatTag;
   WORD nChannels;
   DWORD nSamplesPerSec;
   DWORD nAvgBytesPerSec;
   WORD nBlockAlign;
   WORD wBitsPerSample;
   WORD cbSize;
} WAVEFORMATEX,*PWAVEFORMATEX,*LPWAVEFORMATEX;
typedef const WAVEFORMATEX *LPCWAVEFORMATEX;

#ifdef __cplusplus
extern "C"
{
#endif

MMRESULT waveOutGetVolume(HWAVEOUT,PDWORD);
MMRESULT waveOutSetVolume(HWAVEOUT,DWORD);
MMRESULT waveOutOpen(LPHWAVEOUT,UINT,LPCWAVEFORMATEX,DWORD,DWORD,DWORD);
MMRESULT waveOutClose(HWAVEOUT);
MMRESULT waveOutPrepareHeader(HWAVEOUT,LPWAVEHDR,UINT);
MMRESULT waveOutUnprepareHeader(HWAVEOUT,LPWAVEHDR,UINT);
MMRESULT waveOutWrite(HWAVEOUT,LPWAVEHDR,UINT);
MMRESULT waveOutPause(HWAVEOUT);
MMRESULT waveOutRestart(HWAVEOUT);
MMRESULT waveOutReset(HWAVEOUT);
MMRESULT waveOutBreakLoop(HWAVEOUT);
MMRESULT waveOutGetPosition(HWAVEOUT,LPMMTIME,UINT);
MMRESULT waveOutGetPitch(HWAVEOUT,PDWORD);
MMRESULT waveOutSetPitch(HWAVEOUT,DWORD);
MMRESULT waveOutGetPlaybackRate(HWAVEOUT,PDWORD);
MMRESULT waveOutSetPlaybackRate(HWAVEOUT,DWORD);
MMRESULT waveOutGetID(HWAVEOUT,LPUINT);
MMRESULT waveOutMessage(HWAVEOUT,UINT,DWORD,DWORD);

#ifdef __cplusplus
}
#endif

#endif // __WCEMMSYSTEM_H_
